/*
 * Copyright (C) 2013 Google Inc. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Modified by Octoshape 2015
 */

package octoshape.android;

import android.app.Application;
import android.util.Log;

import octoshape.android.R;
import com.google.sample.cast.refplayer.settings.CastPreference;
import com.google.android.libraries.cast.companionlibrary.cast.VideoCastManager;
import com.google.android.libraries.cast.companionlibrary.utils.Utils;

import octoshape.android.mediaplayer.PlayerManager;
import octoshape.android.stream.StreamManager;
import octoshape.osa2.Problem;
import octoshape.osa2.listeners.ProblemListener;

/**
 * The {@link Application} for this demo application.
 */
public class CastApplication extends Application implements ProblemListener {
    private static final String TAG = "CastApplication";

    private static String sApplicationId;
    private static VideoCastManager sCastMgr = null;
    public static final double VOLUME_INCREMENT = 0.05;
    private static StreamManager streamManager = null;
    private static PlayerManager playerManager = null;

    /*
     * (non-Javadoc)
     * @see android.app.Application#onCreate()
     */
    @Override
    public void onCreate() {
        super.onCreate();
        sApplicationId = getString(R.string.app_id);
        initializeCastManager();

        // initialize the stream manager
        streamManager = StreamManager.initializeStreamManager(this, this, null);

        // initialize the player manager
        playerManager = PlayerManager.initializePlayerManager(getApplicationContext());
    }

    private void initializeCastManager() {
        sCastMgr = VideoCastManager.initialize(getApplicationContext(), sApplicationId, null, getString(R.string.data_channel));
        sCastMgr.enableFeatures(
                VideoCastManager.FEATURE_NOTIFICATION |
                        VideoCastManager.FEATURE_LOCKSCREEN |
                        VideoCastManager.FEATURE_WIFI_RECONNECT |
                        VideoCastManager.FEATURE_CAPTIONS_PREFERENCE |
                        VideoCastManager.FEATURE_DEBUGGING);
    }


    public static VideoCastManager getCastManager() {
        if (sCastMgr == null) {
            throw new IllegalStateException("Application has not been started");
        }
        return sCastMgr;
    }

    public static StreamManager getStreamManager() {
        if (streamManager == null) {
            throw new IllegalStateException("Application has not been started");
        }
        return streamManager;
    }

    public static PlayerManager getPlayerManager() {
        if (playerManager == null) {
            throw new IllegalStateException("Application has not been started");
        }
        return playerManager;
    }

    @Override
    public void gotProblem(final Problem p) {
        Log.e(TAG, "Problem: " + p.toString());
        if (p.getMessage() != null)
            displayMessage(p.getMessage() + "(" + p.getErrorCode() + ")");
    }

    private void displayMessage(final String error) {
        Log.e(TAG, error);
    }
}
